@@ -1,7 +1,6 @@
 use v6;
 
 
-#use IO::Path::More;
 use Linux::Fuser::Procinfo;
 
 =begin pod
@@ -20,11 +19,10 @@ Linux::Fuser - Determine which processes have a file open
 
   my $fuser = Linux::Fuser->new();
 
-  my @procs = $fuser->fuser('foo');
+  my @procs = $fuser.fuser('foo');
 
-  for @procs -> $proc ( @procs )
-  {
-    say $proc->pid(),"\t", $proc->user(),"\n",$proc->cmd();
+  for @procs -> $proc ( @procs ) {
+    say $proc.pid(),"\t", $proc.user(),"\n",$proc->cmd();
   }
 
 =end code
@@ -54,20 +52,33 @@ The class has one method, with two signatures, that does most of the work:
 
 =end pod
 
-class Linux::Fuser:ver<v0.0.6>:auth<github:jonathanstowe> {
+class Linux::Fuser:ver<0.0.11>:auth<github:jonathanstowe>:api<1.> {
     # Shamelessly stolen from IO::Path::More
     # for my own stability
     my role IO::Helper {
         use nqp;
-        method inode() {
-            $*DISTRO.name ne any(<MSWin32 os2 dos NetWare symbian>)
-                && self.e
-                && nqp::p6box_i(nqp::stat(nqp::unbox_s(self.Str), nqp::const::STAT_PLATFORM_INODE))
+
+        has Int $.inode;
+        method inode( --> Int) {
+            if not $!inode.defined {
+                if self.e {
+                    $!inode = nqp::p6box_i(nqp::stat(nqp::unbox_s(self.Str), nqp::const::STAT_PLATFORM_INODE));
+                }
+            }
+            $!inode;
         }
 
-        method device() {
-            self.e && nqp::p6box_i(nqp::stat(nqp::unbox_s(self.Str), nqp::const::STAT_PLATFORM_DEV))
+        has Int $.device;
+
+        method device( --> Int) {
+            if not $!device.defined {
+                if self.e {
+                    $!device = nqp::p6box_i(nqp::stat(nqp::unbox_s(self.Str), nqp::const::STAT_PLATFORM_DEV));
+                }
+            }
+            $!device;
         }
+
         method append (*@nextpaths) {
             my $lastpath = @nextpaths.pop // '';
             self.new($.SPEC.join($.volume, $.SPEC.catdir($.dirname, $.basename, @nextpaths), $lastpath));
@@ -77,26 +88,28 @@ class Linux::Fuser:ver<v0.0.6>:auth<github:jonathanstowe> {
 
     #| Given the path to a file as a String returns a list of L<doc:Linux::Fuser::Procinfo>
     #| objects describing any processes that have the file open
-    multi method fuser (Str $file ) returns Array {
+    multi method fuser (Str $file  --> Array ) {
         self.fuser(IO::Path.new($file));
     }
 
     #| Given the L<doc:IO::Path> that describes the filereturns a list of L<doc:Linux::Fuser::Procinfo>
     #| objects describing any processes that have the file open
-    multi method fuser(IO::Path $file ) returns Array {
+    multi method fuser(IO::Path $file --> Array ) {
         $file does IO::Helper;
         my @procinfo;
         my $device = $file.device;
         my $inode  = $file.inode;
 
-        for dir('/proc', test => /^\d+$/) -> $proc {
-            $proc does IO::Helper;
-            my $fd_dir = $proc.append('fd');
-            if $fd_dir.r {
-                try for $fd_dir.dir(test => /^\d+$/) -> $fd {
-                    $fd does IO::Helper;
-                    if ( self!same_file($file, $fd ) ) {
-                        @procinfo.push(Linux::Fuser::Procinfo.new(proc_file => $proc, fd_file => $fd));
+        if $file.e {
+            for dir('/proc', test => /^\d+$/) -> $proc-file {
+                $proc-file does IO::Helper;
+                my $fd_dir = $proc-file.append('fd');
+                if $fd_dir.r {
+                    for $fd_dir.dir(test => /^\d+$/) -> $fd-file {
+                        $fd-file does IO::Helper;
+                        if ( self!same_file($file, $fd-file ) ) {
+                            @procinfo.push(Linux::Fuser::Procinfo.new(:$proc-file, :$fd-file));
+                        }
                     }
                 }
             }
@@ -104,9 +117,13 @@ class Linux::Fuser:ver<v0.0.6>:auth<github:jonathanstowe> {
         @procinfo;
     }
 
-    method !same_file(IO::Path $left, IO::Path $right) {
+    method !same_file(IO::Path $left, IO::Path $right --> Bool) {
         my Bool $rc = False;
-        if ( ( $left.inode == $right.inode ) && ( $left.device == $right.device )) {
+        sub same($l, $r  --> Bool) {
+            (all($l, $r) ~~ { .defined }) && ( $l == $r );
+        }
+
+        if ( same($left.inode, $right.inode) && same($left.device, $right.device) ) {
             $rc = True;
         }
         return $rc;
